/** @file menuPost.h
 * @brief Declarations generated from menuPost.dbd
 */

#ifndef INC_menuPost_H
#define INC_menuPost_H

#ifndef menuPost_NUM_CHOICES
/** @brief Enumerated type from menu menuPost */
typedef enum {
    menuPost_OnChange               /**< @brief State string "On Change" */,
    menuPost_Always                 /**< @brief State string "Always" */
} menuPost;
/** @brief Number of states defined for menu menuPost */
#define menuPost_NUM_CHOICES 2
#endif


#endif /* INC_menuPost_H */
