/** @file menuSimm.h
 * @brief Declarations generated from menuSimm.dbd
 */

#ifndef INC_menuSimm_H
#define INC_menuSimm_H

#ifndef menuSimm_NUM_CHOICES
/** @brief Enumerated type from menu menuSimm */
typedef enum {
    menuSimmNO                      /**< @brief State string "NO" */,
    menuSimmYES                     /**< @brief State string "YES" */,
    menuSimmRAW                     /**< @brief State string "RAW" */
} menuSimm;
/** @brief Number of states defined for menu menuSimm */
#define menuSimm_NUM_CHOICES 3
#endif


#endif /* INC_menuSimm_H */
