/*************************************************************************\
* Copyright (c) 2007 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#ifndef INC_libComRegister_H
#define INC_libComRegister_H

#include "libComAPI.h"

#ifdef __cplusplus
extern "C" {
#endif

LIBCOM_API void epicsStdCall libComRegister(void);

#ifdef __cplusplus
}
#endif

#endif /* INC_libComRegister_H */
