/*************************************************************************\
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#define EPICS_PRIVATE_API
#include "osiSock.h"

/*
 * epicsSocketUnsentCount ()
 */
int epicsSocketUnsentCount(SOCKET sock) {
    /* not implemented */
    return -1;
}
