/** @file menuPriority.h
 * @brief Declarations generated from menuPriority.dbd
 */

#ifndef INC_menuPriority_H
#define INC_menuPriority_H

#ifndef menuPriority_NUM_CHOICES
/** @brief Enumerated type from menu menuPriority */
typedef enum {
    menuPriorityLOW                 /**< @brief State string "LOW" */,
    menuPriorityMEDIUM              /**< @brief State string "MEDIUM" */,
    menuPriorityHIGH                /**< @brief State string "HIGH" */
} menuPriority;
/** @brief Number of states defined for menu menuPriority */
#define menuPriority_NUM_CHOICES 3
#endif


#endif /* INC_menuPriority_H */
