/*************************************************************************\
* Copyright (c) 2002 The University of Saskatchewan
* SPDX-License-Identifier: EPICS
* EPICS Base is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

/*
 * This header fragment is intended to be included as part of epicsString.h
 */

/*
 * epicsStrtod() for systems with working strtod() routine
 */
#define epicsStrtod strtod
