/*************************************************************************\
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#ifndef osdThreadPvth
#define osdThreadPvth

#ifdef __cplusplus
extern "C" {
#endif

extern HANDLE osdThreadGetTimer(void);

#ifdef __cplusplus
}
#endif

#endif /* osdThreadPvth */
