/** @file menuOmsl.h
 * @brief Declarations generated from menuOmsl.dbd
 */

#ifndef INC_menuOmsl_H
#define INC_menuOmsl_H

#ifndef menuOmsl_NUM_CHOICES
/** @brief Enumerated type from menu menuOmsl */
typedef enum {
    menuOmslsupervisory             /**< @brief State string "supervisory" */,
    menuOmslclosed_loop             /**< @brief State string "closed_loop" */
} menuOmsl;
/** @brief Number of states defined for menu menuOmsl */
#define menuOmsl_NUM_CHOICES 2
#endif


#endif /* INC_menuOmsl_H */
