/*************************************************************************\
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

#include <stdlib.h>

#include <osiFileName.h>

char *epicsGetExecName(void)
{
    return NULL;
}

char *epicsGetExecDir(void)
{
    return NULL;
}
