/** @file menuYesNo.h
 * @brief Declarations generated from menuYesNo.dbd
 */

#ifndef INC_menuYesNo_H
#define INC_menuYesNo_H

#ifndef menuYesNo_NUM_CHOICES
/** @brief Enumerated type from menu menuYesNo */
typedef enum {
    menuYesNoNO                     /**< @brief State string "NO" */,
    menuYesNoYES                    /**< @brief State string "YES" */
} menuYesNo;
/** @brief Number of states defined for menu menuYesNo */
#define menuYesNo_NUM_CHOICES 2
#endif


#endif /* INC_menuYesNo_H */
