/*************************************************************************\
* Copyright (c) 2009 UChicago Argonne LLC, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* SPDX-License-Identifier: EPICS
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
\*************************************************************************/

/* epicsStdioRedirect.h */

/* This file is now obselete, and is likely to be
 * deleted in a future release of EPICS Base.
 *
 * Use the epicsStdio.h header file instead.
 */

#ifndef epicsStdioRedirecth
#define epicsStdioRedirecth

#include <epicsStdio.h>

#endif /* epicsStdioRedirecth */
